#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::deletequery;


use strict;
use warnings;

require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(deletequery);

our $VERSION = '0.01';

#######################################################################
our @ARGVBAACKUP = @ARGV;
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => [ ], 'cmdname'  => "delete_query");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
				 'user'	=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;
our $QueryName = undef;
our $Force = 0;



#########################################################################

#########handle getopt warnings begin########################

sub InitParser
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "delete_query");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$QueryName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub CleanUpParser()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "delete_query");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$QueryName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;

}

#########handle getopt warnings end#####################################

sub handle_help_usage
{
	my $opt = $_[0];
	
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}


sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_force_option
{
	$Force = 1;
}

sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions()
{
	my $ret = undef;
	my $CoreOptRules = {
			'force'   =>	\&handle_force_option 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|login|u|l:s'					=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
		'type:s'							=>	\&handle_recordtype_list			,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}


sub parseForUnknownOptions
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	
	if($lmt == -1) 
	{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<query name>",$CmdInfo{'cmdname'});
	}
	
	if($lmt > 0)
	{
		ThrowSyntaxErrorException("CQCLI_Repeat_QueryName","",$CmdInfo{'cmdname'});
	}
	
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if( $tmpARGV[$cnt] =~ /^(-)(with)/i)
		{
			print "Not supported";
		}
		else
		{
			$QueryName = $tmpARGV[$cnt];
		}
		
	}

	return 1;
}

sub Parse()
{
	try {

		parseForHelpUsage();
		parseForLoginCredentials();
		parseForCoreOptions();
		parseForUnknownOptions();
		
		$JavaReqObj->put("name",$QueryName);
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		return;
	}elsif($CliSession){
		return;
	}else{
		ValidateLoginCredentials();
		return;
	}

}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}

sub Validate()
{
	try {
		ValidateSessionInfo();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub deletequery
{
	my @tmpCmd = @_;
	my $req = undef;
	
	try {
		InitParser(@tmpCmd);
		Parse();
		Validate();
		$req =  GetJavaPayload("delete_query",$JavaReqObj);
		
		if($Force == 0){
			my $promptmsg = GetLocalizedMessage('CQCLI_Prompt_Delete',$QueryName); 
			if(!CheckPromptModeValidity('No_Message') && AskForYN('CQCLI_Prompt_Delete')) {
				SetRequestObject($req);
				return 1;
			}
			else{
				ThrowValidationException('No_Message');
			}
		}
		else {
			SetRequestObject($req);
			return 1;
		}

	}finally {
		CleanUpParser();
	};
}

1;